      SUBROUTINE  COREIO (IFUNCT,IOUNIT,EOFILE,LINE,NWORDS,WORD1,WORD2)
C     (Core I/O Module)
C     This routine utilizes an in-core array as a temporary  file
C     which automatically spills to an external file if it  fills
C     up.
C
C     The space allocated to the array CORE(*) may be adjusted to
C     any convenient  value for  the host  system, provided  that
C     MAXCOR is set accordingly.  A  value MAXCOR = 0 will  cause
C     all records to be on the external file.
C
C     This routine may be used for  only one file at a time.   An
C     initial  explicit  open   call  is   required  for   proper
C     initialization, and a close call must be issued if the file
C     is to be read after writing.
C
C     The I/O functions are selected by the IFUNCT parameter:
C
C          0: Open for input
C          1: Open for output
C          2: Close
C          3: Read (will do implicit open for input if necessary)
C          4: Write (will do implicit open for output if necessary)
C
C     (25-JAN-86)
C  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     EXTERNAL REFERENCES (FUNCTION,SUBROUTINE,COMMON)
C
C     EXTERNAL REFS       F4MVC,       MAX0,        READUA,      WRITUA
C
C  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     INTRINSIC FUNCTIONS
C
      INTEGER             MAX0
C  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     NON-COMMON VARIABLES
C
C     SETS                EOFILE,      KWORDS,      MAXCOR,      MODERD
C     SETS                MODEWR,      WORD1,       WORD2
C
      INTEGER             IFUNCT,      IOUNIT,      KWORDS,      LINE(1)
      INTEGER             MAXCOR,      MODERD,      MODEWR,      NWORDS
      INTEGER             WORD1,       WORD2
      LOGICAL             EOFILE
C
C  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     COMMON BLOCK / COM14  /
C
      INTEGER             WDSIZE
C
      COMMON / COM14  /   WDSIZE
C
C
C     ********** This is used only in COREIO and BLKDAT **********
C
C     Installation on a host which does not save local  variables
C     across calls revealed that  several locals are expected  to
C     be preserved.  A  FORTRAN 77 SAVE  statement would  suffice
C     for this, but to ensure correct operation under FORTRAN 66,
C     we place these in unique COMMON blocks named SAVnn.  It  is
C     conceivable, but  unlikely, that  it  may be  necessary  to
C     place a copy of this COMMON in the MAIN program.
C
C  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     COMMON BLOCK / SAV02  /
C
      INTEGER             CORE,        LSTUSE,      MODE,        RECCOR
      INTEGER             RECPOS,      RECMAX
      LOGICAL             ISOPEN
      COMMON / SAV02  /   CORE(40000), LSTUSE,      MODE,        ISOPEN
      COMMON / SAV02  /   RECCOR,      RECPOS,      RECMAX
C
C
      DATA                MAXCOR    / 40000 /
      DATA                MODERD    / 1 /
      DATA                MODEWR    / 2 /
C
      NX0006=IFUNCT+1
      IF (NX0006.LT.1.OR.NX0006.GT.5) GO TO 20006
      GO TO (20001,20002,20003,20004,20005), NX0006
20001 ASSIGN 20008 TO NPR001
      GO TO 30001
20008 GO TO 20007
20002 ASSIGN 20009 TO NPR002
      GO TO 30002
20009 GO TO 20007
20003 ASSIGN 20010 TO NPR003
      GO TO 30003
20010 GO TO 20007
20004 ASSIGN 20011 TO NPR004
      GO TO 30004
20011 GO TO 20007
20005 ASSIGN 20012 TO NPR005
      GO TO 30005
20012 CONTINUE
20006 CONTINUE
C
20007 RETURN
C
C-----------------------------------------------------------------------
C---- PROCEDURE (Close the File)
30003 LSTUSE = 0
      REWIND IOUNIT
      ISOPEN = .FALSE.
      IF (MODE .EQ. MODEWR) RECMAX = RECPOS
      GO TO 20010
C
C-----------------------------------------------------------------------
C---- PROCEDURE (Open the File)
30001 LSTUSE = 0
      REWIND IOUNIT
      ISOPEN = .TRUE.
      RECPOS = 0
      GO TO NPR001, (20008,20013,20014)
C
C-----------------------------------------------------------------------
C---- PROCEDURE (Open the File for Output)
30002 ASSIGN 20013 TO NPR001
      GO TO 30001
20013 RECMAX = 0
      GO TO NPR002, (20009,20023)
C
C-----------------------------------------------------------------------
C---- PROCEDURE (Read a Record)
30004 MODE = MODERD
      IF (.NOT.(.NOT.ISOPEN)) GO TO 20014
      ASSIGN 20014 TO NPR001
      GO TO 30001
20014 EOFILE = (RECPOS .GE. RECMAX)
      IF (.NOT.(.NOT.EOFILE)) GO TO 20015
      RECPOS = RECPOS + 1
      IF (.NOT.(RECPOS .LE. RECCOR)) GO TO 20018
      KWORDS = CORE(LSTUSE+1)
      WORD1  = CORE(LSTUSE+2)
      WORD2  = CORE(LSTUSE+3)
      CALL F4MVC (CORE(LSTUSE+4),1,LINE,1,KWORDS*WDSIZE)
      LSTUSE = LSTUSE + 3 + KWORDS
      GO TO 20019
20018 READ (IOUNIT,END=20021) KWORDS,WORD1,WORD2
      EOFILE=.FALSE.
      GO TO 20022
20021 EOFILE=.TRUE.
20022 IF (.NOT.EOFILE) CALL READUA (IOUNIT, EOFILE, LINE,
     XKWORDS)
20019 CONTINUE
20020 CONTINUE
20015 CONTINUE
20016 GO TO 20011
C
C-----------------------------------------------------------------------
C---- PROCEDURE (Write a Record)
30005 MODE = MODEWR
      KWORDS = MAX0(1,NWORDS)
      IF (.NOT.(.NOT.ISOPEN)) GO TO 20023
      ASSIGN 20023 TO NPR002
      GO TO 30002
20023 IF (RECPOS .EQ. 0) RECCOR = 0
      IF (.NOT.((RECPOS .EQ. RECCOR) .AND.
     X((LSTUSE+3+KWORDS) .LT. MAXCOR))) GO TO 20024
      CORE(LSTUSE+1) = KWORDS
      CORE(LSTUSE+2) = WORD1
      CORE(LSTUSE+3) = WORD2
      CALL F4MVC (LINE,1,CORE(LSTUSE+4),1,KWORDS*WDSIZE)
      LSTUSE = LSTUSE + 3 + KWORDS
      RECCOR = RECCOR + 1
      GO TO 20025
20024 WRITE (IOUNIT) KWORDS,WORD1,WORD2
      CALL WRITUA (IOUNIT,LINE,KWORDS)
20025 RECPOS = RECPOS + 1
      GO TO 20012
C
      END
